package routers

import (
	"learnsteam/cslms-api/internal/controllers"
	"learnsteam/cslms-api/internal/middleware"
	"learnsteam/cslms-api/internal/repositories"
	"learnsteam/cslms-api/internal/services"

	"github.com/gin-gonic/gin"
	"gorm.io/gorm"
)

func InitQuizRouter(db *gorm.DB, router *gin.Engine) {
	r := NewQuizRouter(db, router)
	r.SetQuizRouter()
}

type QuizRouter interface {
	SetRouter(db *gorm.DB, router *gin.Engine)
}

type quizRouter struct {
	db                  *gorm.DB
	repository          repositories.QuizRepository
	quizPaperRepository repositories.QuizPaperRepository
	service             services.QuizService
	quizPaperService    services.QuizPaperService
	controller          controllers.QuizController
	router              *gin.Engine
}

func NewQuizRouter(db *gorm.DB, router *gin.Engine) *quizRouter {
	repository := repositories.NewQuizRepository(db)
	quizPaperRepository := repositories.NewQuizPaperRepository(db)
	service := services.NewQuizService(repository)
	quizPaperService := services.NewQuizPaperService(quizPaperRepository)
	controller := controllers.NewQuizController(service, quizPaperService)

	return &quizRouter{
		db:                  db,
		repository:          repository,
		quizPaperRepository: quizPaperRepository,
		service:             service,
		quizPaperService:    quizPaperService,
		controller:          controller,
		router:              router,
	}
}

func (r *quizRouter) SetQuizRouter() {
	group := r.router.Group("/quiz")
	group.GET("", middleware.Auth("admin"), r.controller.List)
	group.GET("/:id", middleware.Auth("admin"), r.controller.Find)
	group.POST("", middleware.Auth("admin"), r.controller.Create)
	group.PUT("/:id", middleware.Auth("admin"), r.controller.Update)
	group.DELETE("/:id", middleware.Auth("admin"), r.controller.Delete)
}
